'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _extends = _interopDefault(require('@babel/runtime/helpers/extends'));
var angular = _interopDefault(require('angular'));
require('angular-mocks');
var dom = require('@testing-library/dom');

var mountedContainers = new Set();
var mountedScopes = new Set();

function render(ui, _temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      container = _ref.container,
      _ref$baseElement = _ref.baseElement,
      baseElement = _ref$baseElement === void 0 ? container : _ref$baseElement,
      queries = _ref.queries,
      scope = _ref.scope,
      ignoreUnknownElements = _ref.ignoreUnknownElements;

  if (!baseElement) {
    // default to document.body instead of documentElement to avoid output of potentially-large
    // head elements (such as JSS style blocks) in debug output
    baseElement = document.body;
  }

  if (!container) {
    container = baseElement.appendChild(document.createElement('div'));
  } // we'll add it to the mounted containers regardless of whether it's actually
  // added to document.body so the cleanup method works regardless of whether
  // they're passing us a custom container or not.


  mountedContainers.add(container);
  var $rootScope = getAngularService('$rootScope');
  var $compile = getAngularService('$compile');
  var $scope = $rootScope.$new();
  Object.assign($scope, scope);
  mountedScopes.add($scope);
  var element = $compile(ui)($scope)[0];
  container.appendChild(element);
  $scope.$digest();

  if (!ignoreUnknownElements) {
    assertNoUnknownElements(container);
  }

  return _extends({
    container: container,
    baseElement: baseElement,
    debug: function debug(el) {
      if (el === void 0) {
        el = baseElement;
      }

      return Array.isArray(el) ? // eslint-disable-next-line no-console
      el.forEach(function (e) {
        return console.log(dom.prettyDOM(e));
      }) : // eslint-disable-next-line no-console,
      console.log(dom.prettyDOM(el));
    },
    asFragment: function asFragment() {
      /* istanbul ignore if (jsdom limitation) */
      if (typeof document.createRange === 'function') {
        return document.createRange().createContextualFragment(container.innerHTML);
      }

      var template = document.createElement('template');
      template.innerHTML = container.innerHTML;
      return template.content;
    },
    $scope: $scope,
    unmount: function unmount() {
      $scope.$destroy();
    }
  }, dom.getQueriesForElement(baseElement, queries));
}

function cleanup() {
  mountedScopes.forEach(cleanupScope);
  mountedContainers.forEach(cleanupAtContainer);
} // maybe one day we'll expose this (perhaps even as a utility returned by render).
// but let's wait until someone asks for it.


function cleanupAtContainer(container) {
  if (container.parentNode === document.body) {
    document.body.removeChild(container);
  }

  mountedContainers.delete(container);
}

function cleanupScope(scope) {
  scope.$destroy();
  mountedScopes.delete(scope);
}

function toCamel(s) {
  return s.toLowerCase().replace(/-([a-z])/g, function (_, letter) {
    return letter.toUpperCase();
  });
}

function assertNoUnknownElements(element) {
  var tagName = element.tagName;

  if (tagName.includes('-')) {
    var $injector = getAngularService('$injector');
    var directiveName = toCamel(tagName) + "Directive";

    if (!$injector.has(directiveName)) {
      throw Error("Unknown component/directive \"" + tagName + "\". Are you missing an import?");
    }
  }

  Array.from(element.children).forEach(assertNoUnknownElements);
}

function getAngularService(name) {
  var service;
  angular.mock.inject([name, function (injected) {
    service = injected;
  }]);
  return service;
}

function fireEvent() {
  var $rootScope = getAngularService('$rootScope');
  var result = dom.fireEvent.apply(void 0, arguments);
  $rootScope.$digest();
  return result;
}

Object.keys(dom.fireEvent).forEach(function (key) {
  fireEvent[key] = function () {
    var $rootScope = getAngularService('$rootScope');
    var result = dom.fireEvent[key].apply(dom.fireEvent, arguments);
    $rootScope.$digest();
    return result;
  };
}); // AngularJS maps `mouseEnter` to `mouseOver` and `mouseLeave` to `mouseOut`

fireEvent.mouseEnter = fireEvent.mouseOver;
fireEvent.mouseLeave = fireEvent.mouseOut;

function flush(millis) {
  if (millis === void 0) {
    millis = 50;
  }

  var $browser = getAngularService('$browser');
  var $rootScope = getAngularService('$rootScope');
  var $interval = getAngularService('$interval');
  $interval.flush(millis);
  $browser.defer.flush(millis);
  $rootScope.$digest();
}

function wait(callback, options) {
  if (options === void 0) {
    options = {};
  }

  return dom.wait(function () {
    flush(options.interval);

    if (callback) {
      callback();
    }
  }, options);
}

Object.keys(dom).forEach(function (k) {
  if (k !== 'default') Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () {
      return dom[k];
    }
  });
});
exports.cleanup = cleanup;
exports.fireEvent = fireEvent;
exports.flush = flush;
exports.render = render;
exports.wait = wait;
